/*   
 Arduino code for dynamic playseat 2DOF
 Created 24 May 2011 by Jim Lindblom   SparkFun Electronics https://www.sparkfun.com/products/10182 "Example Code"
 for 2 direction pins drivers like Motomonster, all VNH2SP30 and VNH5019 Motor Driver Carrier
 Created 24 Apr 2012 by Jean David SEDRUE Version betatest26 - 24042012 http://www.gamoover.net/Forums/index.php?topic=25907
 Updated 20 May 2013 by RacingMat in english http://www.xsimulator.net/2dof-wiper-motor-playseat/ in french : http://www.gamoover.net/Forums/index.php?topic=27617
 Updated 30 April 2014 by RacingMat (bug for value below 16 corrected)
 Converted to Sabertooth 15 june 2014 by RacingMat
 */
#include <SabertoothSimplified.h>
#include <SoftwareSerial.h>

////////////////////////////////////////////////////////////////////////////////
#define pwmMax 127 // or less, if you want to lower the maximum motor's speed
//with Sabertooth, the maximum speed is 127 = full forward or -127 full reverse

// defining the range of potentiometer's rotation
const int potMini=208;
const int potMaxi=815;

////////////////////////////////////////////////////////////////////////////////
#define motLeft 1
#define motRight 2
#define potL A0
#define potR A1

////////////////////////////////////////////////////////////////////////////////
//  DECLARATIONS
////////////////////////////////////////////////////////////////////////////////
/* init position value at startup*/
int DataValueL=512; //middle position 0-1024
int DataValueR=512; //middle position 0-1024

SoftwareSerial SWSerial(NOT_A_PIN, 8); //declare the Pin 8 to be used to communicate with Sabertooth
SabertoothSimplified ST(SWSerial); //open a serial COM on previously identified pin

////////////////////////////////////////////////////////////////////////////////
// INITIALIZATION
////////////////////////////////////////////////////////////////////////////////
void setup()
{
  // serial initialization
  Serial.begin(115200); // serial between Arduino and Simtools on RX pin
  SWSerial.begin(38400); // serial between Arduino and Sabertooth on SWSerial pin
  //DIP switches 2, 4 and 5 OFF 

  // stop motors at startup
  ST.motor(motLeft, 0);
  ST.motor(motRight, 0);

}
////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Main Loop ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void loop()
{
  while (1==1) // to get the fastest loop possible
  {
    int sensorL,sensorR;

    readSerialData();   // DataValueR & L contain the last order received (if there is no newer received, the last is kept)
    // the previous order will still be used by the PID regulation MotorMotion Function

    sensorR = analogRead(potR);  // range 0-1024
    sensorL = analogRead(potL);  // range 0-1024

    motorMotion(motRight,sensorR,DataValueR);
    motorMotion(motLeft,sensorL,DataValueL);
  }
}
////////////////////////////////////////////////////////////////////////////////
// Procedure: wait for complete trame
////////////////////////////////////////////////////////////////////////////////
void readSerialData()
{
  byte Data[3]={
    '0','0','0'            };
  // keep this function short, because the loop has to be short to keep the control over the motors

  if (Serial.available()>2){
    //parse the buffer : test if the byte is the first of the order "R"
    Data[0]=Serial.read();
    if (Data[0]=='L'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and that maps the range
      DataValueL=NormalizeData(Data);
    }
    if (Data[0]=='R'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueR=NormalizeData(Data);

    }
  }
  if (Serial.available()>16) Serial.flush();
}
////////////////////////////////////////////////////////
void motorMotion(int numMot,int actualPos,int targetPos)
////////////////////////////////////////////////////////
{
  int Tol=20; // no order to move will be sent to the motor if the target is close to the actual position
  // this prevents short jittering moves
  //could be a parameter read from a pot on an analogic pin
  // the highest value, the calmest the simulator would be (less moves)

  int gap;
  int pwm;
  int brakingDistance=30;

  // security concern : targetPos has to be within the mechanically authorized range
  targetPos=constrain(targetPos,potMini+brakingDistance,potMaxi-brakingDistance);

  gap=abs(targetPos-actualPos);

  if (gap<= Tol) {
    ST.motor(numMot,0); //too near to move: stop motor    
  }
  else {
    // PID : calculates speed according to distance
    pwm=95;
    if (gap>50)   pwm=105;
    if (gap>75)   pwm=115;   
    if (gap>100)  pwm=127;
    pwm=map(pwm, 0, 127, 0, pwmMax);  //adjust the value according to pwmMax for mechanical debugging purpose !

    // if motor is outside from the range, send motor back to the limit !
    // go forward (up) values between 0 to 127
    // if forward needed (((actualPos<potMini) || (actualPos<targetPos))*1-((actualPos>potMaxi) || (actualPos>targetPos))*1)=1
    // if reverse needed it equals -1
     ST.motor(numMot, (((actualPos<potMini) || (actualPos<targetPos))*1-((actualPos>potMaxi) || (actualPos>targetPos))*1)*pwm);

  }
}

////////////////////////////////////////////////////////////////////////////////
// Function: convert Hex to Dec
////////////////////////////////////////////////////////////////////////////////
int NormalizeData(byte x[3])
////////////////////////////////////////////////////////////////////////////////
{
  int result;

  if ((x[2]==13) || (x[2]=='R') || (x[2]=='L'))  //only a LSB and Carrier Return or 'L' or 'R' in case of value below 16 (ie one CHAR and not 2)
  {
    x[2]=x[1];  //move MSB to LSB
    x[1]='0';     //clear MSB
  }
  for (int i=1; i<3; i++)
  {
    if (x[i]>47 && x[i]<58 ){//for xA to xF
      x[i]=x[i]-48;
    }                       
    if (  x[i]>64 && x[i]<71 ){//for x0 to x9
      x[i]=x[i]-55;               
    }
  }
  // map the range from game telemetry (0 <-> 255) to the mechanically authorized range (potMini <-> potMaxi)
  result=map((x[1]*16+x[2]),0,255,potMini,potMaxi);
  return result;
}


